# Copyright (C) 2010-2020, Josef Hahn
#
# This file is part of Ginger.
#
# Ginger is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ginger is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ginger.  If not, see <http://www.gnu.org/licenses/>.

import django.http
import django.core.serializers

import ginger.main.models


def list_feeds(request):
    """
    The handler for url "config/list_feeds".
    """
    return django.http.HttpResponse(
        django.core.serializers.serialize("json",
                                          ginger.main.models.Feed.objects.filter(enabled=True, owner=request.user)))


def disable_feed(request):
    """
    The handler for urls like "config/remove_feed/[some id]".
    """
    feedid = int(request.POST["id"])
    # fetch the feed that is to be removed from the database by the given id from the url
    toberemoved = ginger.main.models.Feed.objects.get(id=int(feedid))
    # check if the logged in user is the owner of that feed. if not, return a simple error message
    if toberemoved.owner != request.user:
        return django.http.HttpResponse("failed")
    # at this point, we are allowed to delete it, so we do so...
    toberemoved.enabled = False
    toberemoved.save()
    # ... and return success to the browser ;)
    return django.http.HttpResponse("Done. Go back!")


def createfeed(name, url, interval, owner):
    feedtag = ginger.main.models.Tag.getnewtag(name, owner)
    f = ginger.main.models.Feed(url=url, name=name, updateInterval=int(interval), owner=owner)
    f.save()
    f.correspondingTags.add(feedtag)


def add_feed(request):
    url = request.POST["url"]
    name = request.POST["name"]
    interval = request.POST["interval"]
    owner = request.user
    createfeed(name, url, interval, owner)
    return django.http.HttpResponse("Done. Go back!")
