/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2010-2020, Josef Hahn

 This file is part of Ginger.

 Ginger is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Ginger is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Ginger.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/

class CloudPanel extends clove.HtmlView {

    constructor(config, domnode) {
        super(clove.utils.applyDefaultsToConfig(config, {
            horizontalStretchAffinity: 1,
        }), domnode);
        this.OnTagClicked = new clove.Event();
    }

    doinit() {
        super.doinit();
        var self = this;
        this.h11 = $("<div class='clovestyle_text_head1 clovestyle_hmargin_small'>Add filter criteria</div>");
        this._tags = $('<div class="taglink tags" style="display: block; overflow: auto;">loading...</div>');
        this._div = $("<div></div>");
        this._div.append(this.h11);
        this._div.append(this._tags);
        this._div.append(this.h12);
        var _get_tags_regularly = undefined;
        _get_tags_regularly = function() {
            if (usercfg)
                self.loadTags(function() {
                    setTimeout(_get_tags_regularly, 30*1000);
                },
                function() {
                    setTimeout(_get_tags_regularly, 30*1000);
                });
        };
        _get_tags_regularly();
        this.setContentRoot(this._div[0]);
    }

    static _tagcountlist_to_tagsizelist(l) {
        var min = Number.MAX_VALUE;
        var max = -1;
        var avg = 0;
        for (var i in l) {
            var tag = l[i];
            var name = tag[0];
            var count = tag[1];
            if (count > max)
                max = count;
            if (count < min)
                min = count;
            avg += count;
        }
        // fontsize = m * count + 12        m=(fontsize-12)/count       m=(30)/count
        var m = 30.0 / (max-min);
        var r = [];
        for (i in l) {
            var tag = l[i];
            var name = tag[0];
            var count = tag[1];
            var size = 12+(count-min)*m;
            r.push([name, size]);
        }
        return r;
    }

    loadTags(onsuccess, onerror) {
        var self = this;
        app.retryAjax({
            url: "tags/get",
            dataType: "json",
            type: "get",
            success: function(data) {
                self._tags.html("");
                var tagsizelist = CloudPanel._tagcountlist_to_tagsizelist(data);
                for (var i in tagsizelist) {
                    var tag = tagsizelist[i];
                    var name = tag[0];
                    var fontsize = tag[1];
                    var lnk = $("<a style='font-size:"+fontsize+"pt;'>"+name+"</a>");
                    lnk.click((function(name) {
                            return function(e) {
                                e.preventDefault();
                                self.OnTagClicked.trigger({widget: self, tag: name});
                            };
                        }(name)));
                    self._tags.append(lnk);
                    self._tags.append(" ");
                }
                if (onsuccess)
                    onsuccess();
            },
            error: function() {
                if (onerror)
                    onerror();
            }
        });
    }

}

