/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2010-2020, Josef Hahn

 This file is part of Ginger.

 Ginger is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Ginger is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Ginger.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


class FeedAdminDialog extends clove.NameScope(clove.Dialog) {

    constructor(config, domnode) {
        super(clove.utils.applyDefaultsToConfig(config, {
            body: {
                rows: [
                    {label: "My Feeds", styleClass: 'clovestyle_text_head2 clovestyle_hmargin_medium clovestyle_vmargin_small'},
                    {view: 'HtmlView', name: 'myfeeds', styleClass: 'feedadmin_feedlistpanel', height: '80pt'},
                    {rows: [
                        {label: "Add New Feed", styleClass: 'clovestyle_text_head2 clovestyle_hmargin_small'},
                        {label: "Fill this form for adding a new feed into the list.\nBefore adding a new feed, you must search for the feed url on the feed's website."},
                        {view: 'Form', sections: [
                            {view: 'EditBox', formSectionLabel: "Name", name: 'edtname'},
                            {view: 'EditBox', formSectionLabel: "Feed URL", name: 'edturl'},
                            {view: 'DropdownBox', formSectionLabel: "Update interval", name: 'edtinterval'},
                        ], styleClass: 'clovestyle_vmargin_small'},
                        {view: 'Button', label: "Add to feed list", name: 'btnaddfeed', strictHorizontalSizing: true, styleClass: 'clovestyle_margin_tiny clovestyle_control_aux'},
                    ], styleClass: 'clovestyle_hmargin_medium clovestyle_vmargin_small'},
                    {view: 'Button', label: "Close", name: 'btnclose', strictHorizontalSizing: true, styleClass: 'clovestyle_marginnotleft_medium'},
                ],
            },
            title: "Feed Management",
        }), domnode);
        this.addStyleClass('gingerdialog');
        this.addStyleClass('feedadmin');
    }

    doinit() {
        super.doinit();
        var self = this;
        this.myfeeds = this.containingNameScope().getByName('myfeeds');
        this.containingNameScope().getByName('btnclose').OnClicked.addHandler(function(e) {
            self.close();
        });
        this.containingNameScope().getByName('btnaddfeed').OnClicked.addHandler(function(e) {
            self._add_feed();
        });
        this._intervals = [[5, "5 minutes"], [15, "15 minutes"], [30, "30 minutes"], [60, "1 hour"], [240, "4 hours"]];
        this._sintervals = [];
        for (var i = 0; i < this._intervals.length; i++)
            this._sintervals.push(this._intervals[i][1]);
        this._dsintervals = clove.utils.arrayToDatasource(this._sintervals);
        var edtinterval = this.containingNameScope().getByName('edtinterval');
        edtinterval.setPopupItems(this._dsintervals);
        edtinterval.setText(this._sintervals[0]);
        this.ndiv = $("<div style='margin: 10pt;'></div>");
        this.ndiv.append('<div class="feedadmin_myfeeds"><table id="feedadmin_myfeeds_table">' +
                    '<tr><th>Name</th><th>URL</th><th>Update interval</th><th></th></tr>' +
                    '</table></div>');
        this.ndiv.append('<div class="feedadmin_nofeeds">You do not have added any feeds so far :(</div>');
        this.edtname = this.containingNameScope().getByName("edtname");
        this.edturl = this.containingNameScope().getByName("edturl");
        this.edtinterval = this.containingNameScope().getByName("edtinterval");
        function _keydown(e) {
            if (e.keyCode == 10 || e.keyCode == 13)
                self._add_feed();
        }
        this.edtname.OnKeyDown.addHandler(_keydown);
        this.edturl.OnKeyDown.addHandler(_keydown);
        this._reload_feedadmin_dialog();
        this.myfeeds.setContentRoot(this.ndiv[0]);
    }

    _reload_feedadmin_dialog(skiploadinganimation) {
        var self = this;
        if (!skiploadinganimation) {
            self.myfeeds.setBusy(true);
            self.ndiv.find(".feedadmin_myfeeds").css("visibility", "collapse");
            self.ndiv.find(".feedadmin_nofeeds").css("visibility", "collapse");
        }
        app.retryAjax({
            url: "config/list_feeds",
            dataType: "json",
            type: "get",
            witherrordialog: true,
            success: function(data) {
                var myfeeds = $("#feedadmin_myfeeds");
                var myfeeds_table = $("#feedadmin_myfeeds_table");
                myfeeds_table.find("tr").filter(".dynamicrow").remove();
                for (var i in data) {
                    var feed = data[i];
                    var btnremove = $("<button style='padding:0 2pt 0 2pt;'>remove</button>");
                    btnremove.click((function(feed) {
                        return function(e) {
                            e.preventDefault();
                            self._disable_feed(feed.pk);
                    }})(feed));
                    var thremove = $("<th></th>");
                    thremove.append(btnremove);
                    var tr = $("<tr class='dynamicrow'></tr>");
                    tr.append("<td>"+feed.fields.name+"</td><td>"+feed.fields.url+"</td><td>"+feed.fields.updateInterval+" min</td>");
                    tr.append(thremove);
                    myfeeds_table.append(tr);
                }
                self.myfeeds.setBusy(false);
                if (!data || !data.length) {
                    self.ndiv.find(".feedadmin_myfeeds").css("visibility", "collapse");
                    self.ndiv.find(".feedadmin_nofeeds").css("visibility", "inherit");
                }
                else {
                    self.ndiv.find(".feedadmin_myfeeds").css("visibility", "inherit");
                    self.ndiv.find(".feedadmin_nofeeds").css("visibility", "collapse");
                }
            },
            error: function() {
                self.close();
            },
        });
    }

    _add_feed() {
        var self = this;
        var name = this.edtname.text();
        var url = this.edturl.text();
        var tinterval = this.edtinterval.text();
        var interval;
        for (var i=0; i<this._intervals.length; i++) {
            if (this._intervals[i][1] == tinterval) {
                interval = this._intervals[i][0];
                break;
            }
        }
        app.ajax({
            url: "config/add_feed",
            dataType: "text",
            type: "post",
            data: {name: name, url: url, interval: interval},
            withloadinganimation: true,
            witherrordialog: true,
            success: function(data) {
                self._reload_feedadmin_dialog(true);
                app.cloudpanel.loadTags();
                app.messagelist.forceFetchMessagesNow();
            },
        });
        this.edtname.setText("");
        this.edturl.setText("");
    }

    _disable_feed(id) {
        var self = this;
        app.retryAjax({
            url: "config/disable_feed",
            dataType: "text",
            data: { id: id },
            type: "post",
            withloadinganimation: true,
            witherrordialog: true,
            success: function(data) {
                self._reload_feedadmin_dialog(true);
            },
        });
    }

}
