/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2010-2020, Josef Hahn

 This file is part of Ginger.

 Ginger is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Ginger is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Ginger.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/

class FilterPanel extends clove.HtmlView {

    constructor(config, domnode) {
        super(config, domnode);
        var self = this;
        this.OnFilterModified = new clove.Event();
    }

    doinit() {
        super.doinit();
        var self = this;
        var div = this._div = $("<div><div class='clovestyle_text_head1 clovestyle_hmargin_small'>Currently filtering by</div><div class='filter_globalactions'></div></div>");
        this._currentview = $("<div></div>");
        div.append(this._currentview);
        this.messagefilters = new Filter();
        if (sessioncfg.touchmode)
            div.addClass("hover");
        else {
            div.hover(function() {
                div.addClass("hover");
            }, function() {
                div.removeClass("hover");
            });
        }
        if (usercfg.powermode) {
            var btnorlayer = $('<button>+ or</button>');
            btnorlayer.click(function(e) {
                e.preventDefault();
                if (self.messagefilters.tofilterstring() != "") {
                    if (self.messagefilters.childs[self.messagefilters.childs.length-1].childs.length > 0) {
                        self.messagefilters.childs.push(new AndFilter([]));
                        self.setFilter(self.messagefilters);
                    }
                }
            });
            div.find(".filter_globalactions").append(btnorlayer);
        }
        var btnstore = $('<button>store filter</button>');
        btnstore.click(function(e){
            e.preventDefault();
            clove.inputDialog({
                question: "If you proceed, your current list of filters will be stored. Please enter a name, so you can find it later on.\n\nIf you enter an existing name, the old one will be overwritten.",
            }).addHandler(function(e) {
                if (e.answer)
                    self.storeFilter(self.messagefilters, e.answer);
            });
        });
        div.find(".filter_globalactions").append(btnstore);
        var btnload = $('<button>load filter</button>');
        btnload.click(function(e) {
            e.preventDefault();
            app.retryAjax({
                url: "filter/list",
                dataType: "json",
                type: "get",
                withloadinganimation: true,
                witherrordialog: true,
                success: function(data) {
                    var filters = [];
                    for (var fii in data.filter)
                        filters.push(data.filter[fii][0]);
                    if (filters.length > 0) {
                        clove.selectionDialog({
                            question: "Please choose the filter you want to load.",
                            choices: filters,
                        }).addHandler(function(e) {
                            if (e.answer !== undefined) {
                                var filtername = filters[e.answer];
                                var filterq;
                                for (var fii in data.filter)
                                    if (data.filter[fii][0] == filtername) {
                                        filterq = data.filter[fii][1];
                                        break;
                                    }
                                self.setFilter(_getFilterForJsonData(JSON.parse(filterq)));
                            }
                        });
                    }
                    else {
                        clove.messageDialog({
                            message: "You don't have stored any filter lists so far.",
                        });
                    }
                },
            });
        });
        div.find(".filter_globalactions").append(btnload);
        var btnclear = $('<button>clear filter</button>');
        btnclear.click(function(e) {
            e.preventDefault();
            self.setFilter();
        });
        div.find(".filter_globalactions").append(btnclear);
        this._update_current_view_text();
        this.setContentRoot(div[0]);
    }

    removeFilter(i) {
        this.messagefilters.childs.splice(i, 1);
        if (this.messagefilters.childs.length == 0)
            this.messagefilters = new Filter();
        this.setFilter(this.messagefilters);
    }

    addFilter(filter) {
        if (this.messagefilters.tofilterstring()) {
            var f = this.messagefilters.childs[this.messagefilters.childs.length-1];
            f.childs.push(filter);
        }
        else
            this.messagefilters = new OrFilter([new AndFilter([filter])]);
        this.setFilter(this.messagefilters);
    }

    _update_current_view_text() {
        this._currentview.html("");
        this._currentview.append(this.messagefilters.toviewhtml(this));
        this.relayout();
    }

    setFilter(filter) {
        if (!filter)
            filter = new Filter();
        var arti = filter;
        if (arti.tofilterstring() != "") {
            if (filter instanceof OrFilter) {
                var artlst = [];
                for(var i=0; i<filter.childs.length; i++) {
                    var c = filter.childs[i];
                    if (c instanceof AndFilter)
                        artlst.push(c);
                    else
                        artlst.push(new AndFilter(c));
                }
                arti = new OrFilter(artlst);
            }
            else {
                arti = filter;
                if (!(arti instanceof AndFilter))
                    arti = new AndFilter([arti]);
                arti = new OrFilter([arti]);
            }
            for (var i=0; i<arti.childs.length; i++) {
                arti.childs[i].parentfilter = arti;
                for (var j=0; j<arti.childs[i].childs.length; j++)
                    arti.childs[i].childs[j].parentfilter = arti.childs[i];
            }
        }
        this.messagefilters = arti;
        this._update_current_view_text();
        this.OnFilterModified.trigger({widget: this});
    }

    messageFilters() {
        return this.messagefilters;
    }

    storeFilter(filter, filtername) {
        app.retryAjax({
            url: "filter/store",
            dataType: "json",
            data: { "name": filtername, "v": filter.tofilterstring() },
            type: "post",
            withloadinganimation: true,
            witherrordialog: true,
        });
    }

}
