/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2010-2020, Josef Hahn

 This file is part of Ginger.

 Ginger is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Ginger is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Ginger.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


class GingerWebClientApplication {

    constructor() {
        sessioncfg.touchmode = navigator.userAgent.includes("Android") ||
                               navigator.userAgent.includes("Phone") ||
                               navigator.userAgent.includes("Tablet") ||
                               navigator.userAgent.includes("Mobile") ||
                               navigator.userAgent.includes("iPad");
    }

    start() {
        var self = this;
        clove.populateUI(function() {
            self.mainview = clove.build({
                view: 'MainView',
                head1: "Ginger RSS Reader",
                icon: clove.Icon.byUrl(sessioncfg.subsite + "static/ginger.png"),
                bodyLeftViewActionLabel: "Show Filters",
                bodyRightViewActionLabel: "Show Messages",
                horizontalStretchAffinity: 1,
                verticalStretchAffinity: 1,
            });
            self.mainview.OnActionTriggered.addHandler(function(e) {
                if (e.action == "logout")
                    self.logout();
                else if (e.action == "feeds")
                    clove.Dialog.show({view: 'FeedAdminDialog'}, {modal: true});
                else if (e.action == "settings")
                    clove.Dialog.show({view: 'SettingsDialog'}, {modal: true});
                else if (e.action == "adminpanel")
                    window.open("admin");
                else if (e.action == "help")
                    window.open(sessioncfg.subsite+"static/readme.pdf", "_blank");
            });
            if (usercfg)
                setTimeout(function() { self._after_login(); });
            else
                self.login();
        });
    }

    login() {
        var self = this;
        var dlgres = clove.Dialog.show({
            view: 'Dialog',
            title: 'Ginger Login',
            mayFocus: true,
            body: {
                rows: [
                    {cols: [
                        {view: 'IconView', icon: clove.Icon.bySymbol(clove.symbols.LockClosed), styleClass: 'clovestyle_margin_small'},
                        {rows: [
                            {label: "Welcome to Ginger!", styleClass: 'clovestyle_text_head1'},
                            {label: "Please enter your login credentials.", styleClass: 'clovestyle_text_major'},
                            {label: "", name: 'lblmessage'},
                        ]},
                    ]},
                    {view: 'Form', sections: [
                        {view: 'EditBox', name: 'edtusername', formSectionLabel: "User Name"},
                        {view: 'PasswordEditBox', name: 'edtpassword', formSectionLabel: "Password"},
                    ], styleClass: 'clovestyle_margin_medium'},
                    {view: 'Button', label: "Login", strictHorizontalSizing: true, name: 'btnlogin', styleClass: 'clovestyle_marginnotleft_medium'},
                ],
                styleClass: 'clovestyle_margin_medium'
            }
        }, {
            modal: true,
        });
        var btnlogin = dlgres.namescope.getByName('btnlogin');
        var lblmessage = dlgres.namescope.getByName('lblmessage');
        var edtusername = dlgres.namescope.getByName('edtusername');
        var edtpassword = dlgres.namescope.getByName('edtpassword');
        function dologin() {
            var username = edtusername.text();
            var password = edtpassword.text();
            self.retryAjax({
                url: "do_login",
                dataType: "json",
                data: { "username": username, "password": password },
                type: "post",
                success: function(data) {
                    var loginfailed = data.loginfailed;
                    if (loginfailed) {
                        lblmessage.setLabel("Invalid login credentials. Please try again.");
                        lblmessage.setStyleClass("clovestyle_margin_tiny clovestyle_text_error");
                    }
                    else {
                        usercfg = data;
                        sessioncfg.freshuser = false;
                        dlgres.widget.close();
                        self._after_login();
                    }
                },
                error: function() {
                    self.showGenericErrorDialog();
                }
            });
        }
        btnlogin.OnClicked.addHandler(function(e) {
            dologin();
        });
        if (sessioncfg.freshuser) {
            lblmessage.setLabel("An initial user is created. Its name is 'admin', password is 'admin'.\nIt has full administrative privileges (can create new users, ...).");
            lblmessage.setStyleClass("clovestyle_margin_tiny");
            edtusername.setText("admin");
            edtpassword.setText("admin");
        }
        edtusername.focus();
        dlgres.widget.OnKeyDown.addHandler(function(e) {
            if (e.keyCode == 10 || e.keyCode == 13)
                dologin();
        });
    }

    _after_login() {
        var self = this;
        clove.populateUI(function() {
            var dsmenu = new clove.NativeDatasource();
            dsmenu.root().addRow([{name: 'logout', label: 'Logout'}]);
            dsmenu.root().addRow([{name: 'feeds', label: 'Feeds'}]);
            dsmenu.root().addRow([{name: 'settings', label: 'Settings'}]);
            if (usercfg.superuser)
                dsmenu.root().addRow([{name: 'adminpanel', label: 'Admin Panel'}]);
            dsmenu.root().addRow([{name: 'help', label: 'Help'}]);
            self.mainview.setActions(dsmenu);
            self.mainview.setBodyRight({view: 'MessageList', name: 'messagelist', emptyText: "<div style='padding:20pt;'><h1>Welcome to Ginger!</h1>Your list is empty. Add some sources for getting more content here. Go to 'Feeds' in the menu bar on the top.</div>"});
            self.messagelist = clove.getByName('messagelist');
            self.mainview.setBodyLeft({rows: [
                {view: 'CloudPanel', name: 'cloudpanel', height: '1px', verticalStretchAffinity: 1, styleClass: 'main_cloudpanel'},
                {view: 'FilterPanel', name: 'filterpanel', styleClass: 'main_filterpanel'},
            ], horizontalStretchAffinity: 1, styleClass: 'main_leftside'});
            self.cloudpanel = clove.getByName('cloudpanel');
            self.filterpanel = clove.getByName('filterpanel');
            self.filterpanel.OnFilterModified.addHandler(function(e){
                var flt = self.filterpanel.messageFilters();
                self.messagelist.setMessageFilters(flt);
                if (flt.tofilterstring() != "")
                    self.messagelist.setEmptyText();
            });
            function _onTagClicked_addFilter(e) {
                self.filterpanel.addFilter(new TagFilter(e.tag));
            }
            self.cloudpanel.OnTagClicked.addHandler(_onTagClicked_addFilter);
            self.messagelist.OnTagClicked.addHandler(_onTagClicked_addFilter);
            self.messagelist.OnMessageCardCreated.addHandler(function(e) {
                self.messagelist.setEmptyText();
            });
            self.setListDirection(usercfg.listdirection, true);
            if (usercfg.defaultfilter)
                self.filterpanel.setFilter(_getFilterForJsonData(JSON.parse(usercfg.defaultfilter)));
            else
                self.filterpanel.setFilter();
            if (usercfg.onuiinitialized) {
                var onuiinitialized = eval("("+usercfg.onuiinitialized+")");
                onuiinitialized();
            }
            self.messagelist.focus();
        });
    }

    logout() {
        var self = this
        var origusercfg = usercfg;
        usercfg = undefined;
        this.mainview.setBodyLeft({label: ''});
        this.mainview.setBodyRight({label: ''});
        if (!origusercfg.onuiinitialized)
            this.login();
        var waitpanel = clove.utils.popup({view:'Widget'}, {modal: true, fullyTransparent: false}).widget;
        this.retryAjax({
            url: "do_logout",
            dataType: "text",
            success: function() {
                waitpanel.remove();
                if (origusercfg.onuiinitialized)
                    window.location.href = window.location.href;
            },
            error: function() {
                waitpanel.remove();
                self.showGenericErrorDialog();
            }
        });
    }

    open_message(url) {
        window.open(url, "_blank");
    }

    showGenericErrorDialog(aerror, btnback) {
        var buttons = (btnback === false) ? [] : [clove.i18n.OK];
        var aaerror = "";
        if (aerror)
            aaerror = aerror;
        var error = "The process failed. " + aaerror + "\n\nIf the problem persists, please contact the system administrator.";
        clove.messageDialog({message: error, title: "There was a problem :(", icon: clove.Icon.bySymbol(clove.symbols.Error), buttons: buttons});
    }


    getListDirection() {
        return usercfg.listdirection;
    }

    setListDirection(v, _norefresh) {
        usercfg.listdirection = (v < 0) ? -1 : 1;
        var reverse = usercfg.listdirection < 0;
        if (this.messagelist.reverselist != reverse) {
            this.messagelist.reverselist = reverse;
            if (!_norefresh)
                this.messagelist.refresh();
        }
    }

    ajax(p) {
        var self = this;
        var cp = new Object();
        var loadinganimation;
        if (p.withloadinganimation)
            loadinganimation = clove.utils.popup({label: ''}, {modal: true});
        for (var i in p)
            cp[i] = p[i];
        cp.url = sessioncfg.subsite + cp.url;
        cp.success = function(data) {
            if (loadinganimation)
                loadinganimation.widget.remove();
            if (p.success)
                p.success(data);
        };
        cp.error = function() {
            if (loadinganimation)
                loadinganimation.widget.remove();
            if (p.witherrordialog)
                self.showGenericErrorDialog();
            if (p.error)
                p.error();
        }
        $.ajax(cp);
    }

    retryAjax(p, tries, _sofar, _loadinganimation) {
        var self = this;
        var sofar = (_sofar === undefined) ? 0 : _sofar;
        var _tries = (tries === undefined) ? 5 : tries;
        var cp = new Object();
        for (var i in p)
            cp[i] = p[i];
        cp.witherrordialog = false;
        cp.withloadinganimation = false;
        var rsuccess = function(data) {
            if (_loadinganimation)
                _loadinganimation.widget.remove();
            if (p.success)
                p.success(data);
        };
        var rerror = function() {
            if (_loadinganimation)
                _loadinganimation.widget.remove();
            if (p.witherrordialog)
                self.showGenericErrorDialog();
            if (p.error)
                p.error();
        };
        if (p.withloadinganimation && !_loadinganimation)
            _loadinganimation = clove.utils.popup({label: ''}, {modal: true});
        if (_tries) {
            cp.error = function() {
                setTimeout(function() {
                    self.retryAjax(p, _tries-1, sofar+1, _loadinganimation);
                }, (sofar+1)*300);
            };
            cp.success = function(data) {
                try {
                    rsuccess(data);
                }
                catch(e) {
                    if (e == "REDOAJAX")
                        setTimeout(function() {
                            self.retryAjax(p, _tries-1, sofar+1, _loadinganimation);
                        }, (sofar+1)*300);
                    else
                        throw e;
                }
            }
        }
        else {
            cp.success = rsuccess;
            cp.error = rerror;
        }
        self.ajax(cp);
    }

}
