/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2010-2020, Josef Hahn

 This file is part of Ginger.

 Ginger is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Ginger is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Ginger.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/

class MessageList extends clove.HtmlView {

    constructor(config, domnode) {
        super(clove.utils.applyDefaultsToConfig(config, {
            mayFocus: true,
            horizontalStretchAffinity: 1,
            verticalStretchAffinity: 1,
            styleClass: 'messagelist',
        }), domnode);
        this.declareProperty('messageFilters');
        this.declareProperty('emptyText', undefined);
        this.OnTagClicked = new clove.Event();
        this.OnMessageCardCreated = new clove.Event();
        this._odiv = $("<div style='position:absolute; left:0; right:0; bottom:0; top:0; overflow:auto;'></div>");
        this._div = $("<div style='position:absolute; left:0; right:0; bottom:0; top:0; overflow:auto;'></div>");
        this._nomsglistdiv = $("<div style='position:absolute; left:0; right:0; bottom:0; top:0; overflow:auto;'></div>");
        this._odiv.append(this._div);
        this._odiv.append(this._nomsglistdiv);
    }

    emptyText_setter(v) {
        this._nomsglistdiv.html(v || "<div style='position:absolute;top:30%;left:20pt;right:20pt;text-align:center;'>There are no results for this filter. You should relax the filtering rules.</div>");
    }

    doinit() {
        super.doinit();
        var self = this;
        this.reverselist = false;
        this.latest_msg_timestamp = undefined;
        this.msg_fetch_request_counter = 0;
        this.togglefavorite_request_counter = 0;
        this.OnKeyDown.addHandler(function(e) {
            if (e.keyCode == 38) {
                // arrow up
                if (self.selected_entry && self.selected_entry.m_prev) {
                    self.selectMessageCard(self.selected_entry.m_prev);
                    self.scrollToMessageCard(self.selected_entry);
                }
                e.originalEvent.preventDefault();
            }
            else if (e.keyCode == 40) {
                // arrow down
                if (self.selected_entry && self.selected_entry.m_next) {
                    self.selectMessageCard(self.selected_entry.m_next);
                    self.scrollToMessageCard(self.selected_entry);
                }
                e.originalEvent.preventDefault();
            }
            else if (e.keyCode == 39) {
                // arrow right
                if (self.selected_entry) {
                    app.open_message(self.selected_entry.msg.url);
                    self.markMessageAsSeen(self.selected_entry);
                }
                e.originalEvent.preventDefault();
            }
            else if (e.keyCode == 46) {
                // del
                if (self.selected_entry)
                    self.deleteMessage(self.selected_entry);
                e.originalEvent.preventDefault();
            }
            else if (e.keyCode == 61 || e.keyCode == 107 || e.keyCode == 171 || e.keyCode == 187) {
                // plus
                if (self.selected_entry)
                    self.openCloseToggleMessageCard(self.selected_entry);
                e.originalEvent.preventDefault();
            }
            else if (e.keyCode == 70) {
                // favorite
                if (self.selected_entry)
                    self.markMessageAsFavorite(self.selected_entry);
                e.originalEvent.preventDefault();
            }
            else if (e.keyCode == 84) {
                // tags
                if (self.selected_entry)
                    self.showtagdialog(self.selected_entry);
                e.originalEvent.preventDefault();
            }
        });
        this._odiv.focusin(function(){
            self.focus(); // otherwise an <a> gets direct focus, which brings trouble when it gets deleted
        });
        this._crawl_and_fetch_regularly = function() {
            if (!self.isAlive())
                return;
            if (usercfg) {
                self._crawlfetching = undefined;
                self._crawl_and_fetch_messages(function() {
                    self._crawlfetching = setTimeout(self._crawl_and_fetch_regularly ,30000);
                });
            }
        };
        setTimeout(this._crawl_and_fetch_regularly, 5000);
        this.setContentRoot(this._odiv[0]);
    }

    forceFetchMessagesNow() {
        var self = this;
        if (this._crawlfetching) {
            clearTimeout(this._crawlfetching);
            this._crawl_and_fetch_regularly();
        }
        else
            setTimeout(function() { self.forceFetchMessagesNow(); }, 100);
    }

    messageFilters_setter() {
        if (this._odiv)
            this.refresh();
    }

    showtagdialog(entry) {
        var self = this;
        var newsMessage = entry.msg;
        var newsId = entry.elemid;
        TaggingDialog.showTaggingDialog(newsMessage, newsId, function(data) {
            self._updatefromdata(entry, data);
        });
    }

    _updatefromdata(entry, data) {
        var nnewsMessage = JSON.parse(data.elements)[0].fields;
        var jtags = JSON.parse(data.tags);
        var tags = new Object();
        for (var ijtag in jtags) {
            var jtag = jtags[ijtag];
            tags[jtag.pk] = jtag.fields.name;
        }
        this._constructMessageCard(nnewsMessage, entry.elemid, tags, entry);
    }

    _appendMessageCard(x) {
        var dd = this._div[0];
        if (!this.reverselist)
            dd.appendChild(x);
        else
            dd.insertBefore(x, dd.childNodes[0]);
        if (!this.last_entry) {
            x.m_next = undefined;
            x.m_prev = undefined;
            this.first_entry = this.last_entry = x;
        }
        else {
            if (!this.reverselist) {
                x.m_prev = this.last_entry;
                this.last_entry.m_next = x;
                x.m_next = undefined;
                this.last_entry = x;
            }
            else {
                x.m_prev = undefined;
                this.first_entry.m_prev = x;
                x.m_next = this.first_entry;
                this.first_entry = x;
            }
        }
        this._nomsglistdiv.css('visibility', 'hidden');
    }

    _constructMessageCard(newsMessage, newsId, tags, useexistingentry) {
        var self = this;
        var vListEntry;
        var createopen;
        if (useexistingentry) {
            createopen = useexistingentry.open;
            vListEntry = useexistingentry;
            $(vListEntry).html("");
        }
        else
            var vListEntry = document.createElement("div");
        vListEntry.open = false;
        vListEntry.elemid = newsId;
        vListEntry.msg = newsMessage;
        newsMessage.favorite = newsMessage.tags.indexOf(this._find_tag_in_list("favorite", tags)) > -1;
        vListEntry.setMainclass = function(cls) {
            var entr = $(this);
            entr.removeClass(this._class);
            this._class = cls;
            entr.addClass(cls);
        };
        if (newsMessage.favorite)
            vListEntry.setMainclass("entryfavorite");
        else
            vListEntry.setMainclass("entrydefault");
        // control panel
        vListEntry.vControl = document.createElement("div");
        vListEntry.vControl.onclick = function(e) {
            e.stopPropagation();
        };
        if (sessioncfg.touchmode)
            $(vListEntry.vControl).addClass("mobileentrycontrolpanel");
        else
            $(vListEntry.vControl).addClass("entrycontrolpanel");
        var lnkOpen = document.createElement("a");
        lnkOpen.innerHTML = "&nbsp;&nbsp;&#10145;";
        lnkOpen.href = newsMessage.url;
        $(lnkOpen).hover(
            function () {
                $(this).addClass("hovered");
            },
            function () {
                $(this).removeClass("hovered");
            }
        );
        $(lnkOpen).click(
            (function(entry) {
                return function() {
                    app.open_message(entry.msg.url);
                    self.markMessageAsSeen(entry);
                    return false;
                }
            })(vListEntry)
        );
        var lnkDel = document.createElement("a");
        lnkDel.innerHTML = "&nbsp;&#10005;&nbsp;";
        $(lnkDel).hover(
            function () {
                $(this).addClass("hovered");
            },
            function () {
                $(this).removeClass("hovered");
            }
        );
        $(lnkDel).click(
            (function(elem) {
                return function() {
                    self.deleteMessage(elem);
                    return false;
                };
            })(vListEntry)
        );
        var lnkFav = document.createElement("a");
        vListEntry.lnkFav = lnkFav;
        lnkFav.className = "enhancedlink";
        if (newsMessage.favorite)
            lnkFav.innerHTML = "&#9733;&nbsp;";
        else
            lnkFav.innerHTML = "&#9734;&nbsp;";
        $(lnkFav).hover(
            function () {
                $(this).addClass("hovered");
            },
            function () {
                $(this).removeClass("hovered");
            }
        );
        $(lnkFav).click(
            (function(elem) {
                return function() {
                    // toggle state in object
                    self.markMessageAsFavorite(elem);
                    return false;
                };
            })(vListEntry)
        );
        var lnkTag = document.createElement("a");
        vListEntry.lnkTag = lnkTag;
        lnkTag.className = "enhancedlink";
        lnkTag.innerHTML = "⚎&nbsp;";
        $(lnkTag).hover(
            function () {
                $(this).addClass("hovered");
            },
            function () {
                $(this).removeClass("hovered");
            }
        );
        $(lnkTag).click(
            (function(elem){
                return function() {
                    self.showtagdialog(elem);
                    return false;
                };
            })(vListEntry)
        );
        vListEntry.vControl.appendChild(lnkFav);
        vListEntry.vControl.appendChild(lnkTag);
        vListEntry.vControl.appendChild(lnkDel);
        vListEntry.vControl.appendChild(lnkOpen);
        if (!sessioncfg.touchmode)
            vListEntry.vControl.style.visibility = "hidden";
        var zwi2 = document.createElement("div");
        zwi2.style.position = "absolute";
        zwi2.style.right = "0";
        zwi2.appendChild(vListEntry.vControl);
        vListEntry.appendChild(zwi2);
        // title
        vListEntry.vListEntryTitle = document.createElement("a");
        vListEntry.vListEntryTitle.setAttribute("name", "title");
        vListEntry.vListEntryTitle.setAttribute("href", newsMessage.url);
        vListEntry.vExpander = document.createElement("span");
        vListEntry.vExpander.style.color = newsMessage.color;
        vListEntry.vInnerTitle = document.createElement("span");
        vListEntry.vExpander.innerHTML = newsMessage.seen ? "&nbsp;◸ " : "&nbsp;◤ ";
        vListEntry.vExpander.className = "expanderpad";
        vListEntry.vInnerTitle.innerHTML = newsMessage.title;
        vListEntry.vListEntryTitle.appendChild(vListEntry.vExpander);
        vListEntry.vListEntryTitle.appendChild(vListEntry.vInnerTitle);
        $(vListEntry).addClass(newsMessage.seen ? "seen" : "unseen");
        var zwi = document.createElement("div");
        zwi.style.padding = "1px";
        vListEntry.appendChild(zwi);
        zwi.appendChild(vListEntry.vListEntryTitle);
        vListEntry.title = newsMessage.title;
        // details
        vListEntry.vListEntryDetails = document.createElement("div");
        vListEntry.vListEntryDetails.setAttribute("name","summary");
        vListEntry.vListEntryDetails.innerHTML = ((newsMessage.summary.length>0) ? newsMessage.summary : "(no summary available)");
        vListEntry.vListEntryDetails.style.display = "none";
        vListEntry.vListEntryDetails.onclick = function(e) {
            e.stopPropagation();
        };
        vListEntry.appendChild(vListEntry.vListEntryDetails);
        if (!useexistingentry) {
            // click handler
            $(vListEntry).click(
                (function(lstentry) {
                    return function() {
                        self.selectMessageCard(lstentry);
                        self.openCloseToggleMessageCard(lstentry);
                        return false;
                    };
                })(vListEntry)
            );
            // hover
            if (!sessioncfg.touchmode) {
                $(vListEntry).hover(
                    (function(lstentry) {
                        return function() {
                            lstentry.vControl.style.visibility = "visible";
                        };
                    })(vListEntry),
                    (function(lstentry) {
                        return function() {
                            lstentry.vControl.style.visibility = "hidden";
                        };
                    })(vListEntry)
                );
            }
        }
        // tags
        vListEntry.vListEntryTags = document.createElement("div");
        vListEntry.vListEntryTags.setAttribute("class", "tags");
        vListEntry.appendChild(vListEntry.vListEntryTags);
        var hastags = false;
        var jvtags = $(vListEntry.vListEntryTags);
        jvtags.html("");
        for (var ti in newsMessage.tags) {
            hastags = true;
            var tag = newsMessage.tags[ti];
            var ttag = tags[tag];
            jvtags.append(" ");
            var tlink = $("<a style='display:block; float:left; margin:2pt;'>"+ttag+"</a>");
            tlink.click(
                (function(ttag) {
                    return function(e) {
                        e.preventDefault();
                        self.OnTagClicked.trigger({widget: self, tag: ttag});
                    };
                })(ttag)
            );
            jvtags.append(tlink);
        }
        if (hastags)
            $(vListEntry.vListEntryTags).css("height", "auto");
        else
            $(vListEntry.vListEntryTags).css("height", "0");
        // footer
        vListEntry.vListEntryFooter = document.createElement("div");
        vListEntry.vListEntryFooter.setAttribute("class", "entryfooter");
        vListEntry.vListEntryFooter.innerHTML = new Date(Date.parse(newsMessage.created)).toLocaleString();
        vListEntry.appendChild(vListEntry.vListEntryFooter);
        if (createopen)
            self.openMessageCard(vListEntry);
        return vListEntry;
    }

    openCloseToggleMessageCard(lstentry) {
        if (lstentry.open)
            this.closeMessageCard(lstentry);
        else
            this.openMessageCard(lstentry);
    }

    openMessageCard(lstentry) {
        if (this.opened_entry)
            this.closeMessageCard(this.opened_entry);
        this.opened_entry = lstentry;
        lstentry.open = true;
        $(lstentry).addClass("expanded");
        this.markMessageAsSeen(lstentry);
        lstentry.vListEntryDetails.style.display = "block";
        lstentry.vListEntryTags.style.display = "block";
        lstentry.vExpander.innerHTML = "&nbsp;◿ ";
    }

    closeMessageCard(lstentry) {
        this.opened_entry = undefined;
        lstentry.open = false;
        $(lstentry).removeClass("expanded");
        lstentry.vListEntryDetails.style.display = "none";
        lstentry.vListEntryTags.style.display = "none";
        lstentry.vExpander.innerHTML = "&nbsp;◸ ";
    }

    markMessageAsSeen(entry) {
        $(entry).removeClass("unseen");
        $(entry).addClass("seen");
        app.retryAjax({
            url: "newsmessages/mark_as_seen",
            type: "post",
            data: { id: entry.elemid },
            dataType: "json",
            success: function(data){},
            error: function() { app.showGenericErrorDialog(); },
        });
    }

    markMessageAsFavorite(entry, val) {
        var self = this;
        var m = entry.msg;
        if (typeof val === 'undefined')
            val = !m.favorite;
        if (val) {
            m.favorite = true;
            entry.setMainclass("entryfavorite");
            entry.lnkFav.innerHTML = " &#9733; ";
        }
        else {
            m.favorite = false;
            entry.setMainclass("entrydefault");
            entry.lnkFav.innerHTML = " &#9734; ";
        }
        var req = ++this.togglefavorite_request_counter;
        app.retryAjax({
            url: val ? "newsmessages/set_tag": "newsmessages/unset_tag",
            dataType: "json",
            type: "post",
            data: { id: entry.elemid, tag: "favorite" },
            witherrordialog: true,
            success: function(data) {
                if (req == self.togglefavorite_request_counter)
                    self._updatefromdata(entry, data);
            },
        });
    }

    deleteMessage(entry) {
        if (entry.msg.favorite) {
            clove.messageDialog({title: "You cannot remove favorites.", message: "Remove favorite status first.", icon: clove.Icon.bySymbol(clove.symbols.Forbidden)});
            return;
        }
        if (entry.m_prev)
            entry.m_prev.m_next = entry.m_next;
        if (entry.m_next)
            entry.m_next.m_prev = entry.m_prev;
        if (entry == this.first_entry)
            this.first_entry = entry.m_next;
        if (entry == this.last_entry)
            this.last_entry = entry.m_prev;
        if (this.selected_entry == entry)
            this.selectMessageCard(entry.m_next ? entry.m_next : entry.m_prev);
        this._div[0].removeChild(entry);
        if (!this.first_entry)
            this._nomsglistdiv.css('visibility', 'visible');
        app.retryAjax({
            url: "newsmessages/remove",
            dataType: "json",
            type: "post",
            data: { id: entry.elemid },
            success: function(data){},
            error: function() { app.showGenericErrorDialog(); },
        });
    }

    selectMessageCard(entry) {
        if (this.selected_entry)
            $(this.selected_entry).removeClass("marked");
        this.selected_entry = entry;
        if (entry)
            $(entry).addClass("marked");
    }

    scrollToMessageCard(msg) {
        var newentry = $(msg);
        var y1 = newentry.offset().top;
        var y2 = y1 + newentry.height();
        var h = this._div.height();
        if (y1 < 50 || y2 > h-50) {
            var yi = (y1+y2) / 2;
            var ys = h / 2;
            var diff = ys - yi;
            this._div[0].scrollTop -= diff;
        }
    }

    _reset_messages() {
        this._div.html("");
        this.selected_entry = this.first_entry = this.last_entry = undefined;
        this.latest_msg_timestamp = undefined;
    }

    _fetch_messages(onsuccess, onfailed, onskipped) {
        var self = this;
        var myrequest = ++this.msg_fetch_request_counter;
        app.retryAjax({
            url: "newsmessages/list",
            dataType: "json",
            data: {
                lasttimestamp: self.latest_msg_timestamp,
                filter: self.config.messageFilters ? self.config.messageFilters.tofilterstring() : undefined,
            },
            success: function(data) {
                if (myrequest == self.msg_fetch_request_counter) {
                    try {
                        var elements = JSON.parse(data.elements);
                    }
                    catch(e) {
                        throw "REDOAJAX";
                    }
                    var jtags = JSON.parse(data.tags);
                    var tags = new Object();
                    for (var ijtag in jtags) {
                        var jtag = jtags[ijtag];
                        tags[jtag.pk] = jtag.fields.name;
                    }
                    for (var i in elements) {
                        var newsMessage = elements[i].fields;
                        var newsId = elements[i].pk;
                        if (!self.latest_msg_timestamp || newsMessage.fetchedAt > self.latest_msg_timestamp)
                            self.latest_msg_timestamp = newsMessage.fetchedAt;
                        var vListEntry = self._constructMessageCard(newsMessage, newsId, tags);
                        self.OnMessageCardCreated.trigger({widget: self, message: newsMessage, card: vListEntry});
                        self._appendMessageCard(vListEntry);
                    }
                    if (elements.length>0 && !self.selected_entry)
                        self.selectMessageCard($(self._div).find("div").first()[0]);
                    if (onsuccess)
                        onsuccess();
                    self.setBusy(false);
                }
                else {
                    if (onskipped)
                        onskipped();
                }
            },
            error: function() {
                if (myrequest == self.msg_fetch_request_counter) {
                    self.setBusy(false);
                    if (onfailed)
                        onfailed();
                }
                else {
                    if (onskipped)
                        onskipped();
                }
            }
        });
    }

    refresh(onsuccess, onfailed) {
        var self = this;
        if (!onfailed)
            onfailed = function() { app.showGenericErrorDialog(undefined, false); };
        this._reset_messages();
        this.setBusy(true);
        this._nomsglistdiv.css('visibility', 'hidden');
        this._fetch_messages(
            function() {
                if (onsuccess)
                    onsuccess();
                self._nomsglistdiv.css('visibility', self.first_entry ? 'hidden' : 'visible');
            },
            function() {
                if (onfailed)
                    onfailed();
                self._nomsglistdiv.css('visibility', self.first_entry ? 'hidden' : 'visible');
            },
            function() {
                if (onsuccess)
                    onsuccess();
            }
        );
    }

    _crawl_and_fetch_messages(oncrawled) {
        var self = this;
        app.ajax({
            url: "crawl/",
            dataType: "text",
            success: function(data) {
                self._fetch_messages(oncrawled, oncrawled, oncrawled);
            },
            error: function() {
                self._fetch_messages(oncrawled, oncrawled, oncrawled);
            }
        });
    }

    _find_tag_in_list(tag, tags) {
        for (var i in tags)
            if (tags[i] == tag)
                return parseInt(i);
    }

}

