/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2010-2020, Josef Hahn

 This file is part of Ginger.

 Ginger is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Ginger is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Ginger.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


class SettingsDialog extends clove.NameScope(clove.Dialog) {

    constructor(config, domnode) {
        super(clove.utils.applyDefaultsToConfig(config, {
            body: {
                rows: [
                    {
                        rows: [
                            {view: 'Expander', name: 'ebasic',
                                body: {view: 'HtmlView', name: 'bbasic', width: '1px', horizontalStretchAffinity: 1},
                                horizontalStretchAffinity: 1, collapsedLabel: "Basic Settings", expandedLabel: "Basic Settings",
                            },
                            {view: 'Expander',
                                body: {view: 'HtmlView', name: 'btagprop', width: '1px', horizontalStretchAffinity: 1},
                                horizontalStretchAffinity: 1, collapsedLabel: "Tag Propagation", expandedLabel: "Tag Propagation",
                            },
                            {view: 'Expander',
                                body: {view: 'HtmlView', name: 'bscripting', width: '1px', horizontalStretchAffinity: 1},
                                horizontalStretchAffinity: 1, collapsedLabel: "Scripting", expandedLabel: "Scripting",
                            },
                        ],
                        styleClass: 'settings_body',
                        width: '540pt',
                    },
                    {view: 'Button', label: 'Close', name: 'btnclose', strictHorizontalSizing: true, styleClass: 'clovestyle_marginnotleft_medium'},
                ],
            },
            title: 'Settings',
        }), domnode);
        this.addStyleClass('gingerdialog');
        this._onnewmessage_default = "def onnewmessage(message):\n"+
                                     "    #summary = message.getsummary().lower()\n"+
                                     "    #if 'space' in summary:\n"+
                                     "    #    message.addtag('deepspace')\n"+
                                     "    #if 'enterprise' in summary:\n"+
                                     "    #    message.addtag('deepspace')\n"+
                                     "    #    message.addtag('enterprise')\n"+
                                     "    #if 'pizza' in summary:\n"+
                                     "    #    message.addtag('archive')\n"+
                                     "    pass\n";
        this._ontagset_default = "def ontagset(message,tag):\n"+
                                 "    #if tag=='deepspace':\n"+
                                 "    #    message.addtag('favorite')\n"+
                                 "    #if tag=='archive':\n"+
                                 "    #    if 'tuna' in message.getsummary().lower():\n"+
                                 "    #        message.delete()\n"+
                                 "    #    else:\n"+
                                 "    #        store_somehow_to_pdf(message.geturl())\n"+
                                 "    pass\n";
        this._onuiinitialized_default = "function onUiInitialized() {\n"+
                                        "    //document.body.style.filter = 'hue-rotate(190deg)';\n"+
                                        "}\n";
    }

    doinit() {
        super.doinit();
        var self = this;
        this.ebasic = this.containingNameScope().getByName('ebasic');
        this.bbasic = this.containingNameScope().getByName('bbasic');
        this.btagprop = this.containingNameScope().getByName('btagprop');
        this.bscripting = this.containingNameScope().getByName('bscripting');
        this.containingNameScope().getByName('btnclose').OnClicked.addHandler(function(e){
            self.close();
        });
        var nbasic = $("<div style='margin: 10pt;'></div>");
        var ntagprop = $("<div style='margin: 10pt;'></div>");
        var nscripting = $("<div style='margin: 10pt;'></div>");
        nbasic.append("<br/>Change some basic settings here. They apply always when you login.<br/><div></div>");
        ntagprop.append("<br/>Define tag propagation rules here, which automatically tag each message " +
                                    "with a certain tag, whenever they get tagged with a certain other tag. " +
                                    "Example: All messages with tag 'gorgonzola' should become tagged with 'bioweapons'. " +
                                    "Tag propagation is an advanced feature.<br/><div></div>");
        nscripting.append("<br/>Write script code here that customize your user experience. This is a very advanced " +
                                    "feature, which you will not get right without reading the manual.<br/><div></div>");
        this._basicsettings = nbasic.find("div");
        this._tagprop = ntagprop.find("div");
        this._script = nscripting.find("div");
        this._basicsettings.append("<div class='settings_loading clovestyle_text_aux clovestyle_hmargin_small'>loading . . .</div><div class='settings_notloading'></div>");
        this._tagprop.append("<div class='settings_loading clovestyle_text_aux clovestyle_hmargin_small'>loading . . .</div><div class='settings_notloading'></div>");
        this._script.append("<div class='settings_loading clovestyle_text_aux clovestyle_hmargin_small'>loading . . .</div><div class='settings_notloading'></div>");
        this._reload_basicsettings();
        this._reload_tagprop();
        this._reload_script();
        this.bbasic.setContentRoot(nbasic[0]);
        this.btagprop.setContentRoot(ntagprop[0]);
        this.bscripting.setContentRoot(nscripting[0]);
        this.ebasic.setIsExpanded(true);
    }

    _reload_basicsettings(skiploadinganimation) {
        var self = this;
        var _setSetting = function(k,v,onsuccess) {
            app.retryAjax({
                url: "config/setvalue?key="+k,
                dataType: "json",
                type: "post",
                data: { value: v },
                withloadinganimation: true,
                witherrordialog: true,
                success: function(data) {
                    self._reload_basicsettings();
                    if (onsuccess)
                        onsuccess();
                }
            });
        };
        self._basicsettings.find(".settings_loading").css("visibility", "collapse");
        self._basicsettings.find(".settings_notloading").css("visibility", "inherit");
        var mydiv = self._basicsettings.find(".settings_notloading");
        mydiv.html("");
        var lst = $("<ul></ul>");
        mydiv.append(lst);
        if (app.getListDirection() == -1) {
            var elemListOrder = $("<li>The message list is sorted from new to old. <button>toggle</button></li>");
            lst.append(elemListOrder);
            elemListOrder.find("button").click(function() {
                app.setListDirection(1);
                _setSetting("reverseSortOrder", "0");
            });
        }
        else {
            var elemListOrder = $("<li>The message list is sorted from old to new. <button>toggle</button></li>");
            lst.append(elemListOrder);
            elemListOrder.find("button").click(function() {
                app.setListDirection(-1);
                _setSetting("reverseSortOrder", "1");
            });
        }
        var elemStoreFilters=$("<li>You can store your current filters as right-after-login default. "+
                               "<button>store my filters as default</button></li>");
        lst.append(elemStoreFilters);
        elemStoreFilters.find("button").click(function() {
            app.retryAjax({
                url: "filter/storedefault",
                dataType: "json",
                type: "post",
                data: { "filterstring": app.filterpanel.messageFilters().tofilterstring() },
                witherrordialog: true,
                success: function(data) {
                    clove.messageDialog({message: "Your current filters are now stored as default."});
                },
            });
        });
        var elemRemoveTags = $("<li>You can remove tags that you don't want to use anymore. " +
                               "<button>remove tags</button></li>");
        lst.append(elemRemoveTags);
        elemRemoveTags.find("button").click(function() {
            var dlgres = clove.Dialog.show({view: 'TaggingDialog', message: "Please choose tags that you want delete now.", tags: [], saveButtonLabel: "Remove", forbidNew: true}, {modal: true});
            dlgres.widget.OnProceed.addHandler(function(e) {
                if (e.tags.length > 0) {
                    app.retryAjax({
                        url: "tags/remove",
                        dataType: "json",
                        type: "post",
                        data: { tags: e.tags },
                        witherrordialog: true,
                        withloadinganimation: true,
                        success: function(data) {
                            app.cloudpanel.loadTags();
                            self._reload_tagprop(true); // they can indirectly be condensed now
                        },
                        error: function() {
                            app.cloudpanel.loadTags();
                            self._reload_tagprop(true); // they can indirectly be condensed now
                        },
                    });
                }
            });
        });
        var elemRemoveFilters = $("<li>You can remove stored filters. <button>remove stored filters</button></li>");
        lst.append(elemRemoveFilters);
        elemRemoveFilters.find("button").click(function() {
            app.retryAjax({
                url: "filter/list",
                dataType: "json",
                type: "get",
                withloadinganimation: true,
                witherrordialog: true,
                success: function(data) {
                    var filters = [];
                    for (var fii in data.filter)
                        filters.push(data.filter[fii][0]);
                    if (filters.length > 0) {
                        clove.selectionDialog({
                            question: "Please choose the filter list you want to remove.",
                            choices: filters,
                        }).addHandler(function(e){
                            if (e.answer !== undefined) {
                                var filtername = filters[e.answer];
                                app.retryAjax({
                                    url: "filter/remove",
                                    dataType: "json",
                                    type: "post",
                                    data: { name: filtername },
                                    withloadinganimation: true,
                                    witherrordialog: true,
                                });
                            }
                        });
                    }
                    else {
                        clove.messageDialog({message: "You don't have stored any filter lists so far."});
                    }
                },
            });
        });
        var elemTogglePowerMode = $(usercfg.powermode ?
            "<li>Currently Ginger offers all features. <button>switch to simple mode</button></li>" :
            "<li>Currently Ginger offers only the basic features. <button>switch to advanced mode</button></li>"
        );
        lst.append(elemTogglePowerMode);
        elemTogglePowerMode.find("button").click(function(){
            app.retryAjax({
                url: "config/setvalue?key=powermode",
                dataType: "json",
                type: "post",
                data: { value: usercfg.powermode ? "0" : "1" },
                withloadinganimation: true,
                witherrordialog: true,
                success: function(data) {
                    location.reload();
                },
            });
        });
        this.bbasic.relayout();
    }

    _reload_tagprop(skiploadinganimation) {
        var phrase11 = "The tag #1 also implies #2.";
        var phrase1n = "The tag #1 also implies #2.";
        var phrasen1 = "The tags #1 also imply #2.";
        var phrasenn = "The tags #1 also imply #2.";
        var self = this;
        if (!skiploadinganimation) {
            self._tagprop.find(".settings_loading").css("visibility", "inherit");
            self._tagprop.find(".settings_notloading").css("visibility", "collapse");
        }
        app.retryAjax({
            url: "tagpropagationrules/get",
            dataType: "json",
            type: "get",
            success: function(data) {
                self._tagprop.find(".settings_loading").css("visibility", "collapse");
                self._tagprop.find(".settings_notloading").css("visibility", "inherit");
                var mydiv = self._tagprop.find(".settings_notloading");
                if (data.length > 0)
                    mydiv.html("Those rules are defined:<br/>");
                else
                    mydiv.html("You have no tag propagation rules defined so far.");
                var mylist = $("<ul></ul>");
                mydiv.append(mylist);
                for (var irule in data) {
                    var rule = data[irule];
                    var myphrase;
                    var iftags = rule[1];
                    var applyalsotags = rule[2];
                    if (iftags.length == 1 && applyalsotags.length == 1)
                        myphrase = phrase11;
                    else if (iftags.length != 1 && applyalsotags.length == 1)
                        myphrase = phrasen1;
                    else if (iftags.length == 1 && applyalsotags.length != 1)
                        myphrase = phrase1n;
                    else
                        myphrase = phrasenn;
                    if (iftags.length != 1) {
                        var n = "'" + iftags.splice(0,iftags.length-1).join("', '") + "' and '" + iftags[iftags.length-1] + "'";
                        myphrase = myphrase.replace("#1", n);
                    }
                    else
                        myphrase = myphrase.replace("#1", "'" + iftags[0] + "'");
                    if (applyalsotags.length != 1) {
                        var n = "'" + applyalsotags.splice(0, applyalsotags.length-1).join("', '") + "' and '" + applyalsotags[applyalsotags.length-1] + "'";
                        myphrase = myphrase.replace("#2", n);
                    }
                    else
                        myphrase = myphrase.replace("#2", "'"+applyalsotags[0]+"'");
                    mylist.append("<li>"+myphrase+" <button>remove</button></li>");
                    mylist.find("button").last().click((function(id) {
                        return function() {
                            app.retryAjax({
                                url: "tagpropagationrules/remove",
                                dataType: "json",
                                data: "id="+id,
                                type: "post",
                                withloadinganimation: true,
                                witherrordialog: true,
                                success: function(data) {
                                    self._reload_tagprop(true);
                                },
                                error: function() {
                                    self._reload_tagprop(true);
                                },
                            });
                        };
                    })(rule[0]));
                }
                var addbtn = $("<button>add a new rule</button>");
                mydiv.append(addbtn);
                addbtn.click(function() {
                    var dlgres1 = clove.Dialog.show({view: 'TaggingDialog', message: "Please choose, which tags should be the condition part of the new rule.", tags: [], saveButtonLabel: "Next"}, {modal: true});
                    dlgres1.widget.OnProceed.addHandler(function(ee1) {
                        if (ee1.tags.length > 0) {
                            var dlgres2 = clove.Dialog.show({view: 'TaggingDialog', message: "Please choose, which tags should be implied/propagated by the new rule.", tags: []}, {modal: true});
                            dlgres2.widget.OnProceed.addHandler(function(ee2) {
                                if (ee2.tags.length>0) {
                                    app.ajax({
                                        url: "tagpropagationrules/add",
                                        dataType: "json",
                                        type: "post",
                                        data: { iftags: ee1.tags, applyalsotags: ee2.tags },
                                        withloadinganimation: true,
                                        witherrordialog: true,
                                        success: function(data) {
                                            self._reload_tagprop(true);
                                        },
                                        error: function() {
                                            self._reload_tagprop(true);
                                        }
                                    });
                                }
                            });
                        }
                    });
                });

                self.btagprop.relayout();
            },
            error: function() {
                self.close();
                app.showGenericErrorDialog();
            },
        });
    }

    _reload_script(skiploadinganimation) {
        var self = this;
        self._script.find(".settings_loading").css("visibility", "collapse");
        self._script.find(".settings_notloading").css("visibility", "inherit");
        var mydiv = self._script.find(".settings_notloading");
        var areaonnewmessage = $("<textarea disabled='disabled' class='settings_scriptingarea'></textarea>");
        var areaontagset = $("<textarea disabled='disabled' class='settings_scriptingarea'></textarea>");
        var areaonuiinitialized = $("<textarea disabled='disabled' class='settings_scriptingarea'></textarea>");
        var btnstoreonnewmessage = $("<button disabled='disabled'>store</button>");
        var btnstoreontagset = $("<button disabled='disabled'>store</button>");
        var btnstoreonuiinitialized = $("<button disabled='disabled'>store</button>");
        mydiv.append("This Python event occurs whenever a new message arrives:<br/>");
        mydiv.append(areaonnewmessage);
        mydiv.append("<br/>");
        mydiv.append(btnstoreonnewmessage);
        mydiv.append("<br/><br/>");
        mydiv.append("This Python event occurs whenever a tag is set on a message:<br/>");
        mydiv.append(areaontagset);
        mydiv.append("<br/>");
        mydiv.append(btnstoreontagset);
        mydiv.append("<br/><br/>");
        mydiv.append("This JavaScript event occurs when the user interface is initialized:<br/>");
        mydiv.append(areaonuiinitialized);
        mydiv.append("<br/>");
        mydiv.append(btnstoreonuiinitialized);
        app.retryAjax({
            url: "config/getvalue?key=onnewmessage",
            dataType: "json",
            type: "get",
            success: function(data) {
                if (!data.value)
                    data = {value: self._onnewmessage_default};
                areaonnewmessage.removeAttr('disabled');
                btnstoreonnewmessage.removeAttr('disabled');
                areaonnewmessage.text(data.value);
            },
            error: function() {
                self.close();
                app.showGenericErrorDialog();
            },
        });
        app.retryAjax({
            url: "config/getvalue?key=ontagset",
            dataType: "json",
            type: "get",
            success: function(data) {
                if (!data.value)
                    data = {value: self._ontagset_default};
                areaontagset.removeAttr('disabled');
                btnstoreontagset.removeAttr('disabled');
                areaontagset.text(data.value);
            },
            error: function() {
                self.close();
                app.showGenericErrorDialog();
            },
        });
        app.retryAjax({
            url: "config/getvalue?key=onuiinitialized",
            dataType: "json",
            type: "get",
            success: function(data) {
                if (!data.value)
                    data = {value: self._onuiinitialized_default};
                areaonuiinitialized.removeAttr('disabled');
                btnstoreonuiinitialized.removeAttr('disabled');
                areaonuiinitialized.text(data.value);
            },
            error: function() {
                self.close();
                app.showGenericErrorDialog();
            },
        });
        btnstoreonnewmessage.click(function() {
            app.retryAjax({
                url: "config/setvalue?key=onnewmessage",
                dataType: "json",
                type: "post",
                data: { value: areaonnewmessage.val() },
                withloadinganimation: true,
                witherrordialog: true,
                success: function(data) {
                    clove.messageDialog({message: "Your script is now active."});
                },
            });
        });
        btnstoreontagset.click(function() {
            app.retryAjax({
                url: "config/setvalue?key=ontagset",
                dataType: "json",
                type: "post",
                data: { value: areaontagset.val() },
                withloadinganimation: true,
                witherrordialog: true,
                success: function(data) {
                    clove.messageDialog({message: "Your script is now active."});
                },
            });
        });
        btnstoreonuiinitialized.click(function() {
            app.retryAjax({
                url: "config/setvalue?key=onuiinitialized",
                dataType: "json",
                type: "post",
                data: { value: areaonuiinitialized.val() },
                withloadinganimation: true,
                witherrordialog: true,
                success: function(data) {
                    clove.messageDialog({message: "Your script is active after next login."});
                },
            });
        });
        this.bscripting.relayout();
    }

}
