/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2010-2020, Josef Hahn

 This file is part of Ginger.

 Ginger is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Ginger is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Ginger.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/


class TaggingDialog extends clove.NameScope(clove.Dialog) {

    constructor(config, domnode) {
        super(clove.utils.applyDefaultsToConfig(config, {
            body: {view: 'HtmlView', name: 'body', width: '500pt', height: '400pt'},
            mayFocus: true,
            busy: true,
        }), domnode);
        this.declareProperty('tags', []);
        this.declareProperty('message', "");
        this.declareProperty('saveButtonLabel', "Save");
        this.declareProperty('forbidNew', false);
        this.OnProceed = new clove.Event();
        this.addStyleClass('gingerdialog');
    }

    doinit() {
        super.doinit();
        var self = this;
        this.body = this.containingNameScope().getByName('body');
        this.ndiv = $("<div></div>");
        this.OnKeyDown.addHandler(function(e) {
            if (e.keyCode == 27) {
                e.originalEvent.preventDefault();
                self.close();
            }
            else if (e.keyCode == 13 || e.keyCode == 10) {
                e.originalEvent.preventDefault();
                self.close();
                self.OnProceed.trigger({widget: self, tags: self.newmytags});
            }
        });
        var nmessage = $("<div class='taggingdialog_part'></div>");
        nmessage.text(this.config.message);
        this.ndiv.append(nmessage);
        this.ndiv.append("<div class='taggingdialog_part'>Check/Uncheck tags in the following list.</div>");
        this.geometrydetection = $("<div class='taggingdialog_part'></div>");
        this.ndiv.append(self.geometrydetection);
        this.name2link = [];
        this.tagcloud = $("<div class='tags' style='display:block;position:absolute;overflow:auto; left:30pt;right:30pt;'></div>");
        this.ndiv.append(this.tagcloud);
        this.btnpanel = $("<div class='taggingdialog_part' style='position:absolute; bottom:0;left:30pt;right:30pt;'></div>");
        var btncancel = $("<button class='importantbutton'>Cancel</button>");
        this.ndiv.append(self.btnpanel);
        btncancel.click(function(e) {
            e.preventDefault();
            self.close();
        });
        this.btnpanel.append(btncancel);
        var btnok = $("<button class='importantbutton'></button>");
        btnok.text(this.config.saveButtonLabel);
        btnok.click(function(e) {
            e.preventDefault();
            self.close();
            self.OnProceed.trigger({widget: self, tags: self.newmytags});
        });
        this.btnpanel.append(btnok);
        if (!this.config.forbidNew) {
            var panelnewtag = $("<span style='margin-left:40pt;'><input type='text' style='width:100pt'/><button class='importantbutton'>Add tag</button></span>");
            var newtextfield = panelnewtag.find("input");
            panelnewtag.find("button").click(function(e) {
                e.preventDefault();
                var tag = newtextfield[0].value.trim();
                if (tag.length > 0) {
                    app.retryAjax({
                        url: "tags/add",
                        dataType: "text",
                        data: { tag: tag },
                        type: "post",
                        success: function(data) {
                            self.newmytags.push(data);
                            self._loadtags();
                        },
                        error: function() {
                            app.showGenericErrorDialog();
                        }
                    });
                }
                newtextfield[0].value = "";
            });
            this.panelnewtag = panelnewtag;
            this.btnpanel.append(panelnewtag);
            newtextfield.focus();
            newtextfield.keydown(function(e) {
                if (e.which == 10 || e.which == 13) {
                    var txt = newtextfield.val();
                    if (txt == "") {
                        self.close();
                        self.OnProceed.trigger({widget: self, tags: self.newmytags});
                    }
                    else
                        panelnewtag.find("button").click();
                    return false;
                }
            });
        }
        else
            this.panelnewtag = $();
        this.newmytags = [];
        for (var ti in self.config.tags)
            this.newmytags.push(self.config.tags[ti]);
        this.body.setContentRoot(this.ndiv[0]);
        this._resize = function() {
            var t = self.geometrydetection.position().top;
            var b = self.btnpanel.height()+20;
            self.tagcloud.css("top", t+"px");
            self.tagcloud.css("bottom", b+"px");
        }
        this.OnResized.addHandler(function() {
            self._resize();
        });
        this._resize();
        this._loadtags();
    }

    _loadtags() {
        var self = this;
        self.panelnewtag.find("button").attr("disabled", "disabled");
        app.retryAjax({
            url: "tags/get",
            dataType: "json",
            type: "get",
            success: function(data) {
                self.setBusy(false);
                self.panelnewtag.find("button").removeAttr("disabled");
                self.tagcloud.html("");
                var alltagsizelist = CloudPanel._tagcountlist_to_tagsizelist(data);
                for (var ti in alltagsizelist) {
                    var tag = alltagsizelist[ti];
                    var name = tag[0];
                    var size = tag[1];
                    self._addtaglink(name, size, self.newmytags.indexOf(name)>-1);
                }
                self.body.relayout();
            },
            error: function() {
                app.showGenericErrorDialog();
                self.close();
            }
        });
    }

    _addtaglink(name, size, activated) {
        var lnk = $("<a style='display:block; float:left; margin:3pt; font-size:"+size+"pt;'>"+name+"</a>");
        var self = this;
        lnk.click((function(lnk, name) {
            return function(e) {
                e.preventDefault();
                if (self.newmytags.indexOf(name) > -1) {
                    self.newmytags.splice(self.newmytags.indexOf(name), 1);
                    lnk.removeClass('activated');
                }
                else {
                    self.newmytags.push(name);
                    lnk.addClass('activated');
                }
            };
        })(lnk, name));
        if(activated)
            lnk.addClass("activated");
        this.tagcloud.append(lnk);
        this.name2link[name] = lnk;
    }

    static showTaggingDialog(newsMessage, newsId, onsuccess) {
        app.retryAjax({
            url: "newsmessages/list?lasttimestamp=&filter=" + encodeURIComponent(new IdFilter(newsId).tofilterstring()),
            dataType: "json",
            type: "get",
            withloadinganimation: true,
            witherrordialog: true,
            success: function(data) {
                var jtags = JSON.parse(data.tags);
                var tags = new Object();
                for (var ijtag in jtags) {
                    var jtag = jtags[ijtag];
                    tags[jtag.pk] = jtag.fields.name;
                }
                var mytags = [];
                var nnm = JSON.parse(data.elements)[0].fields;
                for (var iitag in nnm.tags) {
                    var itag = nnm.tags[iitag];
                    var tag = tags[itag];
                    mytags.push(tag);
                }
                var dlgres = clove.Dialog.show({view: 'TaggingDialog', title: 'Tag this message.', tags: mytags}, {modal: true});
                dlgres.widget.OnProceed.addHandler(function(e) {
                    var tagdata = new Object();
                    for (var i in e.tags)
                        tagdata["tag"+i] = e.tags[i];
                    tagdata.id = newsId;
                    app.retryAjax({
                        url: "newsmessages/set_tags",
                        dataType: "json",
                        type: "post",
                        data: tagdata,
                        withloadinganimation: true,
                        witherrordialog: true,
                        success: function(data) {
                            app.cloudpanel.loadTags();
                            if (onsuccess)
                                onsuccess(data);
                        },
                        error: function() {
                            app.cloudpanel.loadTags();
                        },
                    });
                });
            },
        });

    }

}
